package com.androdome.polyterra;

import java.io.File;
import java.io.IOException;
import java.util.UUID;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.abstraction.Level;
import com.cliffblaster.game.api.event.PermissionRequestEvent;
import com.cliffblaster.game.level.gen.GenProperties;
import com.cliffblaster.game.level.gen.GenType;
import com.cliffblaster.game.net.ServerHandler;

public class CreateCommand extends Command
{

	final MainPlugin plugin;

	public CreateCommand(MainPlugin plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		PermissionRequestEvent e = plugin.getManager().requestPermission(caller, "polyterra.admin.create");
		return (e.wasHandled() && e.hasPermission()) || caller == null || caller.isAdmin;
	}

	private static long generateSeed(String[] seedParts, int length) {
		String seed = "";
		for(int i = length; i < seedParts.length; i++)
			seed += seedParts[i] + " ";
		seed = seed.substring(0, seed.length()-1);
		try
		{
			return Long.parseLong(seed);
		}
		catch (Exception ex)
		{
			return UUID.nameUUIDFromBytes(seed.getBytes()).getMostSignificantBits();
		}
	}

	@Override
	public boolean execute(String[] args, ServerHandler caller) {
		if (args.length > 1)
		{
			try
			{
				String name = args[1];
				if(new File(this.plugin.worldsDir, name + ".dat").exists())
				{
					sendMessage(caller, "A level with that name already exists!", ChatColor.RED);
					return true;
				}
				if(name.contains("?"))
				{
					sendMessage(caller, "That filename is not valid!", ChatColor.RED);
					return true;
				}
				try
				{
					if(!new File(this.plugin.worldsDir, name + ".dat").createNewFile())
					{
						sendMessage(caller, "That filename is not valid!", ChatColor.RED);
						return true;
					}
					else new File(this.plugin.worldsDir, name + ".dat").delete();
				}
				catch (IOException e)
				{
					sendMessage(caller, "That filename is not valid!", ChatColor.RED);
					return true;
				}
				GenType generator = GenType.standardValue();
				int width = 256, height = 64, length = 256;
				GenProperties genProperties = new GenProperties();
				if (args.length > 2)
				{
					generator = GenType.fromNameOrId(args[2], null);
					if (generator == null)
					{
						sendMessage(caller, "That level generator is not valid!", ChatColor.RED);
						return true;
					}
				}
				if (args.length > 3) width = Integer.parseInt(args[3]);
				if (args.length > 4) height = Integer.parseInt(args[4]);
				if (args.length > 5) length = Integer.parseInt(args[5]);
				if (args.length > 6)
				{
					String property = args[6].trim();
					while (property.length() < 6)
						property += "1";
					genProperties.hasPlants.setValue(property.charAt(0) == '1');
					genProperties.hasLiquids.setValue(property.charAt(1) == '1');
					genProperties.hasSediment.setValue(property.charAt(2) == '1');
					genProperties.hasOres.setValue(property.charAt(3) == '1');
					genProperties.hasCaves.setValue(property.charAt(4) == '1');
					genProperties.hasBounds.setValue(property.charAt(5) == '1');
				}
				genProperties.height = height;
				genProperties.length = length;
				genProperties.width = width;
				if (args.length > 7) genProperties.seed = generateSeed(args, 7);
				genProperties.generator = generator;

				boolean go = true;
				if (width % 16 != 0 || length % 16 != 0)
				{
					sendMessage(caller, "The level size must be a multiple of 16!", ChatColor.RED);
					go = false;
				}
				if (width < 16 || length < 16)
				{
					sendMessage(caller, "The level size must be at least 16x16!", ChatColor.RED);
					go = false;
				}
				if (width > 10000 || length > 10000 || height > 10000)
				{
					sendMessage(caller, "The level sizes must be at most 10000!", ChatColor.RED);
					go = false;
				}
				if (height < 48)
				{
					sendMessage(caller, "The level height must be at least 48!", ChatColor.RED);
					go = false;
				}
				if (go)
				{
					sendMessage(caller, "Creating Level...", ChatColor.GREEN);
					new Thread(new CreationThread(this, caller, name, genProperties)).start();
					return true;
				}
			}
			catch (NumberFormatException ex)
			{
				sendMessage(caller, "The level size is not a valid integer!", ChatColor.RED);
			}

		}
		return false;
	}

	@Override
	public String getHelpDesc() {
		// TODO Auto-generated method stub
		return "Creates a new level";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " <name> [generator=0.0.11a] [width=256] [height=64]", ChatColor.CYAN);
		sendMessage(caller, "   [length=256] [properties=111111] [seed]", ChatColor.CYAN);
		sendMessage(caller, "Properties are a boolean value, 1 is true and 0 is false", ChatColor.CYAN);
		sendMessage(caller, "Properties, in order: Plants, Water, Sands, Ores, Caves, HasBounds", ChatColor.CYAN);
	}

	void sendMessagePublic(ServerHandler caller, String message, ChatColor color) {
		this.sendMessage(caller, message, color);
	}
}

class CreationThread implements Runnable
{
	private final CreateCommand command;
	private final ServerHandler caller;
	private final String name;
	private final GenProperties genProp;

	public CreationThread(CreateCommand createCommand, ServerHandler caller, String name, GenProperties genProp) {
		this.command = createCommand;
		this.caller = caller;
		this.name = name;
		this.genProp = genProp;
	}

	public void run() {
		try
		{
			Level level = new Level(genProp, command.plugin.getServer());
			if (command.plugin.addNewLevel(name, level)) command.sendMessagePublic(caller, "Created!", ChatColor.GREEN);
			else command.sendMessagePublic(caller, "Failed to apply level!", ChatColor.GREEN);
		}
		catch (Exception ex)
		{
			command.sendMessagePublic(caller, "Failed to generate level!", ChatColor.RED);
		}
	}

}
