package com.androdome.polyterra;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import com.cliffblaster.game.api.abstraction.Level;
import com.cliffblaster.game.api.abstraction.LevelLoader;
import com.cliffblaster.game.api.abstraction.Server;
import com.cliffblaster.game.net.ServerHandler;

public class LevelData
{
	public final Level level;
	boolean levelIsCompressed = false;
	boolean levelCanCompress = false;
	private final File worldDir;
	public final String levelName;
	public String levelDisplayName;
	public boolean ticks = true;
	public boolean autoload = false;
	public String motd = "A nice level";
	private final MainPlugin plugin;
	ArrayList<Integer> allowedBlocks = null;
	Set<ServerHandler> connectedPlayers = Collections.synchronizedSet(new HashSet<ServerHandler>());

	public LevelData(Level level, File worldDir, String levelName, MainPlugin mainPlugin) {
		this.levelDisplayName = levelName;
		this.level = level;
		this.worldDir = worldDir;
		this.levelName = levelName;
		this.plugin = mainPlugin;
	}

	public void compress() throws InterruptedException {
		synchronized (level)
		{
			if (levelCanCompress && !levelIsCompressed) try
			{
				this.plugin.startSaving(this);
				this.plugin.waitForSaveThread();
				levelIsCompressed = true;
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				DataOutputStream dos = new DataOutputStream(new GZIPOutputStream(baos));
				dos.write(level.blocks);
				dos.close();
				baos.close();
				level.blocks = baos.toByteArray();
			}
			catch (IOException ex)
			{
				levelIsCompressed = false;
			}
		}
	}

	public void decompress() {
		synchronized (level)
		{
			if (levelIsCompressed) try
			{
				ByteArrayInputStream bais = new ByteArrayInputStream(level.blocks);
				DataInputStream dis = new DataInputStream(new GZIPInputStream(bais));
				byte[] blockBuffer = new byte[level.width * level.height * level.length];
				dis.readFully(blockBuffer);
				level.blocks = blockBuffer;
				dis.close();
				bais.close();
				levelIsCompressed = false;
			}
			catch (IOException ex)
			{
			}
		}
	}

	public static LevelData load(String levelName, File worldDir, MainPlugin mainPlugin) {
		File dataToLoad = new File(worldDir, levelName + ".properties");
		Properties properties = new Properties();
		FileInputStream fis = null;
		try
		{
			fis = new FileInputStream(dataToLoad);
			properties.load(fis);
			fis.close();
			fis = null;
		}
		catch (Exception ex)
		{
			if (fis != null)
			{
				try
				{
					fis.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}
		File levelToLoad = new File(worldDir, levelName + ".dat");
		Level level = LevelLoader.loadFile(levelToLoad);
		if (level != null)
		{
			level.name = levelName;
			LevelData data = new LevelData(level, worldDir, levelName, mainPlugin);
			data.motd = properties.getProperty("motd", data.motd);
			data.levelDisplayName = properties.getProperty("disp", levelName);
			try
			{
				data.autoload = Boolean.parseBoolean(properties.getProperty("autoload", data.autoload + ""));
				data.ticks = Boolean.parseBoolean(properties.getProperty("ticks", data.ticks + ""));
				data.levelCanCompress = Boolean.parseBoolean(properties.getProperty("cancompress", data.levelCanCompress + ""));
			}
			catch (Exception ex)
			{
			}
			data.readAllowedBlocks();
			if(data.levelCanCompress){
				try
				{
					data.compress();
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
			return data;
		}
		return null;
	}

	public void readAllowedBlocks() {
		File allowedBlocks = new File(worldDir, levelName + ".ald");
		if (allowedBlocks.exists())
		{
			BufferedReader reader = null;
			try
			{
				ArrayList<Integer> allowedList = new ArrayList<Integer>();
				reader = new BufferedReader(new FileReader(allowedBlocks));
				String read;
				while ((read = reader.readLine()) != null)
				{
					if (read.trim().length() > 0)
					{
						allowedList.add(Integer.parseInt(read.replace("\r", "").replace("\n", "")));
					}
				}
				this.allowedBlocks = allowedList;
			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
			catch (NumberFormatException e)
			{
				e.printStackTrace();
			}
			finally
			{
				if (reader != null)
				{
					try
					{
						reader.close();
					}
					catch (IOException e)
					{
						e.printStackTrace();
					}
				}
			}
		}
	}

	public void saveAllowedBlocks(File worldDir) {
		if (allowedBlocks == null) return;
		File allowedBlocks = new File(worldDir, levelName + ".ald");
		BufferedWriter writer = null;
		try
		{
			writer = new BufferedWriter(new FileWriter(allowedBlocks));
			for (Integer i : this.allowedBlocks)
			{
				writer.write(i + "");
				writer.newLine();
			}
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		catch (NumberFormatException e)
		{
			e.printStackTrace();
		}
		finally
		{
			if (writer != null)
			{
				try
				{
					writer.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}
	}

	public boolean store() {
		if (levelIsCompressed) return false;
		Server.log.info("[PolyTerra] Saving level " + levelName + "...");
		File dataToSave = new File(worldDir, levelName + ".properties");
		Properties properties = new Properties();
		properties.setProperty("motd", motd);
		properties.setProperty("disp", levelDisplayName);
		properties.setProperty("ticks", ticks + "");
		properties.setProperty("autoload", autoload + "");
		properties.setProperty("cancompress", levelCanCompress + "");
		FileOutputStream fos = null;
		try
		{
			fos = new FileOutputStream(dataToSave);
			properties.store(fos, "PolyTerra level data");
		}
		catch (Exception ed)
		{
			return false;
		}
		finally
		{
			if (fos != null)
			{
				try
				{
					fos.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}
		this.saveAllowedBlocks(worldDir);
		File levelToSave = new File(worldDir, levelName + ".dat");
		if (LevelLoader.storeFile(levelToSave, level))
		{
			Server.log.info("[PolyTerra] Saved!");
			return true;
		}
		else
		{
			Server.log.info("[PolyTerra] Failed!");
			return false;
		}
	}
}
