package com.androdome.polyterra;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.JavaPlugin;
import com.cliffblaster.game.api.RegisteredEvent;
import com.cliffblaster.game.net.ServerHandler;
import com.cliffblaster.game.net.packet.Packet9Chat;
import com.cliffblaster.game.api.abstraction.Level;
import com.cliffblaster.game.api.abstraction.Server;
import com.cliffblaster.game.api.event.Event.Priority;
import com.cliffblaster.game.api.event.Event.Type;
import com.cliffblaster.game.api.event.PlayerDisconnectEvent;
import com.cliffblaster.game.api.event.PlayerEventListener;
import com.cliffblaster.game.api.event.ServerEventListener;
import com.cliffblaster.game.api.event.ServerSaveEvent;
import com.cliffblaster.game.api.event.ServerTickEvent;


public class MainPlugin extends JavaPlugin
{
	ConcurrentHashMap<String, LevelData> nameToLevel = new ConcurrentHashMap<String, LevelData>();
	ConcurrentHashMap<LevelData, String> levelToName = new ConcurrentHashMap<LevelData, String>();
	ConcurrentHashMap<ServerHandler, RegisteredEvent> events = new ConcurrentHashMap<ServerHandler, RegisteredEvent>();
	public File worldsDir;
	private SaveThread saveThread = null;

	@Override
	public void init() {
		worldsDir = new File(this.getPluginDirectory(), "worlds");
		worldsDir.mkdir();
		this.registerCommand("create", new CreateCommand(this), true);
		this.registerCommand("load", new LoadCommand(this), true);
		this.registerCommand("unload", new UnloadCommand(this), true);
		Command command = new JoinCommand(this);
		this.registerCommand("join", command, true);
		this.registerCommand("j", command, false);
		this.registerCommand("return", new ReturnCommand(this), true);
		this.registerCommand("setproperty", new SetPropertyCommand(this), true);
		this.registerCommand("getproperties", new GetPropertiesCommand(this), true);
		this.registerCommand("delete", new DeleteCommand(this), true);
		this.registerCommand("levels", new ListCommand(this), true);
		this.registerCommand("rename", new RenameCommand(this), true);
		ServerListener listener = new ServerListener(this);
		this.registerEvent(Type.SERVER_TICK, listener, Priority.LOWEST);
		this.registerEvent(Type.SERVER_SAVE, listener, Priority.LOWEST);
		this.registerEvent(Type.PLAYER_DISCONNECT, new PlayerDisconnectListener(this), Priority.LOWEST);
		for (File worldFile : worldsDir.listFiles())
		{
			if (worldFile.getName().endsWith(".properties"))
			{
				Properties properties = new Properties();
				FileInputStream fis = null;
				try
				{
					fis = new FileInputStream(worldFile);
					properties.load(fis);
					fis.close();
					fis = null;
				}
				catch (Exception ex)
				{
					if (fis != null)
					{
						try
						{
							fis.close();
						}
						catch (IOException e)
						{
							e.printStackTrace();
						}
					}
				}
				try
				{
					if (Boolean.parseBoolean(properties.getProperty("autoload", "false")))
					{
						loadLevel(worldFile.getName().replace(".properties", ""));
					}
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
				}
			}
		}
		Server.log.info("[PolyTerra] Starting PolyTerra...");
	}

	@Override
	public void destroy() {
		try
		{
			waitForSaveThread();
			Server.log.info("[PolyTerra] Saving on exit...");
			startSaving();
			waitForSaveThread();
			Server.log.info("[PolyTerra] Done!");
		}
		catch (InterruptedException ex)
		{
			ex.printStackTrace();
		}
	}

	synchronized public void waitForSaveThread() throws InterruptedException {
		synchronized (this)
		{
			if (saveThread != null) saveThread.join();
		}
	}

	synchronized public void startSaving() {
		startSaving(null);
	}

	synchronized public void startSaving(LevelData levelData) {
		synchronized (this)
		{
			if (saveThread == null)
			{
				saveThread = new SaveThread(this, levelData);
				saveThread.start();
			}
		}
	}

	void clearSaveThread() {
		saveThread = null;
	}

	public boolean addLevel(String name, LevelData data) {

		name = name.toLowerCase();
		if (nameToLevel.get(name) != null) return false;
		this.nameToLevel.put(name, data);
		this.levelToName.put(data, name);
		Server.log.info("[PolyTerra] Loaded level " + name);
		return true;

	}

	public boolean loadLevel(String name) {
		name = name.toLowerCase();
		if (this.nameToLevel.get(name) != null) return false;
		LevelData data = LevelData.load(name, worldsDir, this);
		if (data == null) return false;
		addLevel(name, data);
		return true;
	}

	public boolean addNewLevel(String name, Level newLevel) {
		name = name.toLowerCase();
		LevelData data = new LevelData(newLevel, worldsDir, name, this);
		if (data.store()) return true;
		else return false;
	}

	public boolean unloadLevel(String name) {
		name = name.toLowerCase();

		final LevelData data = this.nameToLevel.get(name);
		synchronized (Server.getPlayers())
		{
			for (ServerHandler handler : Server.getPlayers())
			{
				final ServerHandler player = (ServerHandler) handler;
				if (player.getLevel().equals(data.level))
				{
					new Thread() {
						public void run() {
							getServer();
							player.setLevel(data.level, "Joining server hub", Server.getDesc());
							Server.broadcast(new Packet9Chat((player.getFullPlayerName().startsWith("&") ? "" : ChatColor.PURPLE) + player.getFullPlayerName() + " " + ChatColor.PURPLE + "has joined the level!"), null, player.getLevel());
							player.sendPacket(new Packet9Chat(ChatColor.ORANGE + "The world you were in was unloaded,"));
							player.sendPacket(new Packet9Chat(ChatColor.ORANGE + " so you were brought back to the server hub"));
						}
					}.start();
				}
			}
		}
		if (data == null) return false;
		if (!data.store()) return false;
		this.nameToLevel.remove(name);
		this.levelToName.remove(data);

		Server.log.info("Unloaded level " + name);
		System.gc();

		return true;
	}

	public void clearEvents(ServerHandler caller) {
		RegisteredEvent event = events.remove(caller);
		if (event != null) unregisterEvent(event);
	}

	public void addEvent(ServerHandler handler, RegisteredEvent event) {
		clearEvents(handler);
		events.put(handler, event);
	}

	public LevelData getDataFromLevel(Level level) {

		for (LevelData data : this.nameToLevel.values())
		{
			if (data.level.equals(level)) return data;
		}

		return null;
	}

}

class ServerListener extends ServerEventListener
{
	MainPlugin plugin;

	ServerListener(MainPlugin plugin) {
		this.plugin = plugin;
	}

	public void onServerTick(ServerTickEvent serverTickEvent) {

		for (LevelData level : plugin.nameToLevel.values())
		{
			if (level.ticks && !level.levelIsCompressed && level.connectedPlayers.size() > 0)
			{
				level.level.tick();
			}
		}

	}

	public void onServerSave(ServerSaveEvent serverSaveEvent) {
		plugin.startSaving();
	}
}

class PlayerListener extends PlayerEventListener
{
	MainPlugin plugin;

	PlayerListener(MainPlugin plugin) {
		this.plugin = plugin;
	}

	public void onPlayerDisconnected(PlayerDisconnectEvent playerDisconnect) {
		plugin.clearEvents(playerDisconnect.getHandler());
	}
}

class PlayerDisconnectListener extends PlayerEventListener
{
	MainPlugin plugin;

	public PlayerDisconnectListener(MainPlugin mainPlugin) {
		this.plugin = mainPlugin;
	}

	public void onPlayerDisconnected(PlayerDisconnectEvent event) {
		ServerHandler handler = (ServerHandler) event.getHandler();
		if (handler.getLevel() == null) return;

		LevelData data = plugin.getDataFromLevel(handler.getLevel());
		if (data != null)
		{
			data.connectedPlayers.remove(handler);
			if (data.levelCanCompress && data.connectedPlayers.size() == 0)
			{
				try
				{
					data.compress();
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}
	}
}

class SaveThread extends Thread
{
	MainPlugin thisPlugin;
	LevelData dataToSave;

	public SaveThread(MainPlugin mainPlugin, LevelData levelData) {
		thisPlugin = mainPlugin;
		dataToSave = levelData;
	}

	public void run() {
		if (dataToSave == null)

		for (LevelData level : thisPlugin.nameToLevel.values())
		{
			level.store();
		}

		else dataToSave.store();
		System.gc();
		this.thisPlugin.clearSaveThread();
	}
}